# -*- coding: utf-8 -*-

from typing import List

from PyQt5.QtCore import QByteArray
from PyQt5.QtWidgets import QWidget, QDialogButtonBox, QAbstractButton

from libfly_csp.interactive_process_dialog import InteractiveProcessDialog
from libastra_csp.utils import Utils
from astra_csp_cryptopro.cprocsp_process import CprocspProcess
from fly_csp_cryptopro.graphic_cryptopro_settings import graphic_cryptopro_settings


class CprocspProcessDialog(InteractiveProcessDialog):
    def __init__(self, process: CprocspProcess, parent: QWidget = None) -> None:
        super().__init__(process=process, parent=parent)
        self.__setup()

    @staticmethod
    def __question_buttons(array: QByteArray) -> List[QDialogButtonBox.StandardButton]:
        found_buttons = []  # type: List[QDialogButtonBox.StandardButton]
        lines = Utils.to_utf8(array).split('\n')
        for line in lines:
            for button, data in (graphic_cryptopro_settings.process_question_keys or {}).items():
                for alias in data['aliases']:
                    if line.find(alias) >= 0:
                        found_buttons.append(button)
                        break

        return found_buttons

    def __setup(self) -> None:
        self.answersButtonBox.clicked.connect(self.__on_buttonbox_clicked)
        self._process.readyReadStandardOutput.connect(self.__on_process_ready_read_standard_output)

    def __on_process_ready_read_standard_output(self) -> None:
        output = self._process.output()
        if len(output) > 0:
            self.waitLabel.setVisible(False)
            self.textEdit.setVisible(True)
        else:
            self.waitLabel.setVisible(True)
            self.textEdit.setVisible(False)

        self.textEdit.setText(output)
        self.__check_question_keys()

    def __check_question_keys(self) -> None:
        answer_buttons = []
        for button in self.__question_buttons(self._process.last_standard_output):
            if button in (graphic_cryptopro_settings.process_question_keys or {}).keys():
                answer_buttons.append(button)

        buttons = QDialogButtonBox.StandardButtons()
        for button in answer_buttons:
            buttons |= button
        self.answersButtonBox.setStandardButtons(buttons)

        if answer_buttons:
            self.show()

    def __on_buttonbox_clicked(self, button: QAbstractButton) -> None:
        key = self.answersButtonBox.standardButton(button)
        if key in (graphic_cryptopro_settings.process_question_keys or {}):
            answer = graphic_cryptopro_settings.process_question_keys[key]['answer']
            self._process.write(QByteArray(len(answer), answer))
            self.answersButtonBox.setStandardButtons(QDialogButtonBox.StandardButtons())
