# -*- coding: utf-8 -*-

import sys

from typing import List

from PyQt5.QtCore import QCoreApplication, QCommandLineParser

from libfly_csp.plugins.fly_csp_plugin import FlyCspPlugin
from libfly_csp.translator import GraphicTranslator
from libfly_csp.common_functions import setup_parser
from fly_csp_cryptopro.mainwindow import MainWindow


class CryproPro:
    @staticmethod
    def load(application: QCoreApplication) -> None:
        application.id = 'fly-csp-cryptopro'
        application.init()

        GraphicTranslator.load_all_translations()
        GraphicTranslator.load_translation(directory='cryptopro', prefix='astra_csp_cryptopro_')
        GraphicTranslator.load_translation(directory='cryptopro', prefix='fly_csp_cryptopro_')

    @staticmethod
    def setup_parser(parser: QCommandLineParser) -> None:
        setup_parser(parser)

    @staticmethod
    def exec(application: QCoreApplication, parser: QCommandLineParser, plugins: List[FlyCspPlugin]) -> None:
        if application.is_running:
            application.sendMessage('\t'.join(parser.positionalArguments()))
            sys.exit(0)

        window = MainWindow()

        window.show()
        window.initialize()
        for plugin in plugins:
            plugin.setup_main_window(window)
        window.perform_input_files(parser.positionalArguments())
        window.start()

        application.activation_window = window
        application.messageReceived.connect(window.on_application_message_received)
