#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys

import fly_csp_plugins

from typing import List

from PyQt5.QtCore import QCoreApplication, QCommandLineParser
from PyQt5.QtWidgets import QMessageBox

from libastra_csp.plugins.plugin_loader import PluginLoader
from libfly_csp.plugins.fly_csp_plugin import FlyCspPlugin
from libfly_csp.single_application import QtSingleApplication
from fly_csp_cryptopro.cryptopro_main import CryproPro

translate = QCoreApplication.translate


def main() -> None:
    application = QtSingleApplication(sys.argv)

    try:
        plugins = PluginLoader.load(module=fly_csp_plugins, plugin_type=FlyCspPlugin)  # type: List[FlyCspPlugin]
        for plugin in plugins:
            plugin.load()

        CryproPro.load(application)

        parser = QCommandLineParser()
        CryproPro.setup_parser(parser)
        for plugin in plugins:
            plugin.setup_parser(parser)
        parser.process(application)

        CryproPro.exec(application, parser, plugins)
        for plugin in plugins:
            plugin.exec(application, parser)

        sys.exit(application.exec_())
    except Exception as exception:
        QMessageBox.critical(None, translate('text', 'Critical error'), str(exception))


if __name__ == '__main__':
    main()
